//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Dialogs.hpp>
#include <IniFiles.hpp>
#include "App_Document.h"
#include "App_OptionsDialog.h"
#include "App_ProgressDialog.h"
#include "App_ScanTasks.h"
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
#include "Moa_Sets.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_document {
//---------------------------------------------------------------------------
void __fastcall TDocument::Initialize()
{
	try {
		FEntries = new TScanEntries();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocument::Uninitialize()
{
	delete FEntries;
	FEntries = NULL;
}
//---------------------------------------------------------------------------
TScanAspects __fastcall TDocument::GetAspects()
{
	return FAspects;
}
//---------------------------------------------------------------------------
void __fastcall TDocument::SetAspects(const TScanAspects& AAspects)
{
	if(AAspects == Aspects) {
		return;
	}
	FAspects = AAspects;
	DoAspectsChange();
}
//---------------------------------------------------------------------------
void __fastcall TDocument::DoAspectsChange()
{
	if(OnAspectsChange != NULL) {
		OnAspectsChange(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocument::TDocument() :
	TModel(),
	FEntries(NULL),
	FAspects(TScanAspects() << saSize << saType << saHash),
	FRememberSettings(false),
	FOnAspectsChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocument::~TDocument()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocument::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		// Load first
		RememberSettings = ASettings->ReadBool(
			LSection,
			"RememberSettings",
			RememberSettings);

		if(RememberSettings) {
			Aspects = TSetUtilities<TScanAspect, saDirectory, saBinaries>::FromString(ASettings->ReadString(
				LSection,
				"Aspects",
				TSetUtilities<TScanAspect, saDirectory, saBinaries>::ToString(Aspects)));
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocument::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection;
	{
		if(RememberSettings) {
			ASettings->WriteString(
				LSection,
				"Aspects",
				TSetUtilities<TScanAspect, saDirectory, saBinaries>::ToString(Aspects));
		}
		ASettings->WriteBool(
			LSection,
			"RememberSettings",
			RememberSettings);
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Initialize()
{
	try {
		FSaveResultsDialog = new TSaveDialog(NULL);
		FSaveResultsDialog->Title = "Save Results";
		FSaveResultsDialog->DefaultExt = "txt";
		FSaveResultsDialog->Filter = "Text (*.txt)|*.txt|All (*.*)|*.*";
		FSaveResultsDialog->FilterIndex = 1;
		FSaveResultsDialog->Options = TOpenOptions(FSaveResultsDialog->Options)
			<< ofOverwritePrompt;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Uninitialize()
{
	delete FSaveResultsDialog;
	FSaveResultsDialog = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::TaskStatus(TObject* ASender, const AnsiString& AMessage)
{
	AppProgressDialog->Message = AMessage;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::TaskProgress(TObject* ASender, int ACurrent, int AAll)
{
	AppProgressDialog->ShowProgress(ACurrent, AAll);
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::TaskTerminate(TObject* ASender)
{
	AppProgressDialog->Finish();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::TaskException(TObject* ASender, Exception* AException)
{
	if(Application->OnException != NULL) {
		Application->OnException(Application, AException);
	}
	else {
		Application->ShowException(AException);
	}
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::TDocumentCommands(TDocument* ADocument) :
	TCommands(),
	FDocument(ADocument),
	FSaveResultsDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TDocumentCommands::~TDocumentCommands()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::Scan()
{
	if(!CanScan()) {
		HandleFeatureUnavailable();
	}
	if(Document->Entries->CountByPriority(spPrimary) == 0) {
		Application->MessageBox(
			"Specify primary entries to perform a scan task.",
			Application->Title.c_str(),
			MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
		Abort();
	}
	TAutoPointer<TScanTask> LTask = new TScanTask(Document);
	TAppProgressDialog::EnsureCreated();
	AppProgressDialog->Caption = "Scan";
	AppProgressDialog->Reset();
	AppProgressDialog->OnCancel = LTask->HandleCancel;
	LTask->FreeOnTerminate = true;
	LTask->OnStatus = TaskStatus;
	LTask->OnProgress = TaskProgress;
	LTask->OnTerminate = TaskTerminate;
	LTask->OnException = TaskException;
	LTask->Perform();
	LTask.Release();
	AppProgressDialog->Execute();
	if(!AppProgressDialog->IsCompleted()) {
		Abort();
	}
	Application->MessageBox(
		AnsiString().sprintf(
			"%s duplicate files in %s groups were found.",
			TCommon::FormatInteger(Document->Entries->CountDuplicate()).c_str(),
			TCommon::FormatInteger(Document->Entries->CountGroups()).c_str()).c_str(),
		"Scan",
		MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanScan()
{
	return Document->Entries->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::ShowOptions()
{
	if(!CanShowOptions()) {
		HandleFeatureUnavailable();
	}
	TAppOptionsDialog::EnsureCreated();
	TAppOptions LValues = AppOptionsDialog->Values;
	LValues.Aspects = Document->Aspects;
	LValues.RememberSettings = Document->RememberSettings;
	AppOptionsDialog->Values = LValues;
	if(!AppOptionsDialog->Execute()) {
		Abort();
	}
	LValues = AppOptionsDialog->Values;
	Document->Aspects = LValues.Aspects;
	Document->RememberSettings = LValues.RememberSettings;
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanShowOptions()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SaveResults()
{
	SaveResultsDialog->FileName = "Dupy Scan Results.txt";
	if(!SaveResultsDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TStrings> LLines;
	{
		TAutoPointer<TObjectList> LTable;
		{
			int LMaxGroup = 0;
			for(int LI = 0; LI < Document->Entries->Count; ++LI) {
				if(Document->Entries->Items[LI]->Group > LMaxGroup) {
					LMaxGroup = Document->Entries->Items[LI]->Group;
				}
			}
			LTable = new TObjectList();
			LTable->Count = LMaxGroup + 1;
		}
		for(int LI = 0; LI < Document->Entries->Count; ++LI) {
			int LGroup = Document->Entries->Items[LI]->Group;
			if(LGroup <= 0) {
				continue;
			}
			if(LTable->Items[LGroup] == NULL) {
				LTable->Items[LGroup] = new TStringList();
			}
			static_cast<TStrings*>(LTable->Items[LGroup])->Add(Document->Entries->Items[LI]->Path);
		}
		LLines = new TStringList();
		LLines->Capacity = LTable->Count;
		for(int LI = 0; LI < LTable->Count; ++LI) {
			if(LTable->Items[LI] != NULL) {
				LLines->Add(static_cast<TPathList*>(LTable->Items[LI])->Text);
			}
		}
	}
	try {
		LLines->SaveToFile(SaveResultsDialog->FileName);
	}
	catch(EStreamError& LException) {
		throw EAppError(AnsiString().sprintf(
			"The program could not save the results to the file \"%s\".",
			SaveResultsDialog->FileName.c_str()));
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDocumentCommands::CanSaveResults()
{
	return Document->Entries->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::LoadSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".AddRegularFilesDialog";
	{
		SaveResultsDialog->FilterIndex = ASettings->ReadInteger(
			LSection,
			"FilterIndex",
			SaveResultsDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TDocumentCommands::SaveSettings(TMemIniFile* ASettings, const AnsiString& ASection)
{
	AnsiString LSection = ASection + ".SaveResultsDialog";
	{
		ASettings->WriteInteger(
			LSection,
			"FilterIndex",
			SaveResultsDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_document
//---------------------------------------------------------------------------

